@@ -55,6 +55,8 @@ use std::mem;
 pub trait ClMem {
     fn get(&self) -> cl_mem;
 
+    fn get_mut(&mut self) -> cl_mem;
+
     fn mem_type(&self) -> Result<cl_mem_object_type> {
         Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_TYPE)?.to_uint())
     }
@@ -101,7 +103,7 @@ pub trait ClMem {
         Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_USES_SVM_POINTER)?.to_uint())
     }
 
-    // CL_VERSION_3_0
+    /// CL_VERSION_3_0
     fn properties(&self) -> Result<Vec<cl_ulong>> {
         Ok(memory::get_mem_object_info(self.get(), MemInfo::CL_MEM_PROPERTIES)?.to_vec_ulong())
     }
@@ -128,6 +130,10 @@ impl<T> ClMem for Buffer<T> {
     fn get(&self) -> cl_mem {
         self.buffer
     }
+
+    fn get_mut(&mut self) -> cl_mem {
+        self.buffer
+    }
 }
 
 impl<T> Drop for Buffer<T> {
@@ -282,6 +288,10 @@ impl ClMem for Image {
     fn get(&self) -> cl_mem {
         self.image
     }
+
+    fn get_mut(&mut self) -> cl_mem {
+        self.image
+    }
 }
 
 impl Drop for Image {
@@ -312,6 +322,7 @@ impl Image {
     ///
     /// returns a Result containing the new OpenCL image object
     /// or the error code from the OpenCL C API function.
+    #[cfg(feature = "CL_VERSION_1_2")]
     pub fn create(
         context: &Context,
         flags: cl_mem_flags,
@@ -690,6 +701,7 @@ impl Sampler {
         Ok(Sampler::new(sampler))
     }
 
+    #[cfg(feature = "CL_VERSION_2_0")]
     pub fn create_with_properties(
         context: &Context,
         properties: *const cl_sampler_properties,
@@ -757,23 +769,31 @@ impl Sampler {
 /// Has methods to return information from calls to clGetPipeInfo with the
 /// appropriate parameters.  
 /// Implements the Drop trait to call release_mem_object when the object is dropped.
+#[cfg(feature = "CL_VERSION_2_0")]
 #[derive(Debug)]
 pub struct Pipe {
     pipe: cl_mem,
 }
 
+#[cfg(feature = "CL_VERSION_2_0")]
 impl ClMem for Pipe {
     fn get(&self) -> cl_mem {
         self.pipe
     }
+
+    fn get_mut(&mut self) -> cl_mem {
+        self.pipe
+    }
 }
 
+#[cfg(feature = "CL_VERSION_2_0")]
 impl Drop for Pipe {
     fn drop(&mut self) {
         memory::release_mem_object(self.get()).expect("Error: clReleaseMemObject");
     }
 }
 
+#[cfg(feature = "CL_VERSION_2_0")]
 impl Pipe {
     pub fn new(pipe: cl_mem) -> Pipe {
         Pipe { pipe }
