@@ -24,6 +24,8 @@ use cl3::memory::{
 };
 use cl3::types::{cl_device_svm_capabilities, cl_svm_mem_flags, cl_uint};
 use libc::c_void;
+#[cfg(feature = "serde")]
+use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::fmt;
 use std::fmt::Debug;
 use std::iter::IntoIterator;
@@ -31,6 +33,8 @@ use std::marker::PhantomData;
 use std::mem;
 use std::ops::{Deref, DerefMut};
 use std::ptr;
+#[allow(unused_imports)]
+use std::result;
 
 struct SvmRawVec<'a, T> {
     ptr: *mut T,
@@ -161,7 +165,7 @@ impl<'a, T> Drop for SvmRawVec<'a, T> {
 ///
 /// Intel provided an excellent overview of Shared Virtual Memory here:
 /// [OpenCL 2.0 Shared Virtual Memory Overview](https://software.intel.com/content/www/us/en/develop/articles/opencl-20-shared-virtual-memory-overview.html).  
-/// A PDF version is available here: [SVM Overview](../docs/svmoverview.pdf).
+/// A PDF version is available here: [SVM Overview](https://github.com/kenba/opencl3/blob/main/docs/svmoverview.pdf).
 ///
 /// To summarise, a CL_DEVICE_SVM_COARSE_GRAIN_BUFFER requires the SVM to be *mapped*
 /// before being read or written by the host and *unmapped* afterward, while
@@ -487,6 +491,24 @@ impl<'a, T: Debug> fmt::Debug for SvmVec<'a, T> {
     }
 }
 
+#[cfg(feature = "serde")]
+impl<'a, T> Serialize for SvmVec<'a, T>
+where
+    T: Serialize,
+{
+    fn serialize<S>(&self, serializer: S) -> result::Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut seq = serializer.serialize_seq(Some(self.len()))?;
+
+        for element in self.iter() {
+            seq.serialize_element(element)?;
+        }
+        seq.end()
+    }
+}
+
 struct RawValIter<T> {
     start: *const T,
     end: *const T,
